unit SHUtilityProxy;

/// Routines from SHLibrary that are needed for the demonstration so that it
/// can be distributed without the library

interface

uses
  {Delphi} Windows;

const
  PROCESS_NOT_START=DWORD($FFFFFFFE);

(*LIBHELP 11101126 \DP SHCompleteProcess
\SK SHCreateProcess
Runs a command and waits for it to complete.
{\par\par\f1 function SHCompleteProcess(const ACommandLine:string; var AExitCode:cardinal;
\  AWaitInterval:integer):dword;}
\par
\T2
\intbl ACommandLine\cell The command to run\cell\row
\intbl AWaitInterval\cell A Timeout interval in milliseconds.  The constant INFINITE
\  may be used to mean just that\cell\row
\intbl AExitCode\cell The Exit Value returned by the command\cell\row
intbl Result\cell
\  PROCESS_NOT_START - Creation of process failed, to get extended error information,
\  call GetLastError
\line WAIT_FAILED - Error, to get extended error information, call GetLastError
\line WAIT_ABANDONED - Another Error, see windows SDK Help for WaitForSingleObject
\line WAIT_OBJECT_0	- Success
\line WAIT_TIMEOUT - Timed out
\cell\row
{\i\par\par This wrapper function was written before 64 bit and unicode.  In its
\  current form it may fail when trying to start a 64 bit program and when ACommandLine
\  is a unicode string.  See Microsoft help for CompleteProcess().}
*)
function SHCompleteProcess(const ACommandLine:string; var AExitCode:cardinal;
 AWaitInterval:dword=INFINITE; ANewConsole: boolean = false):dword;

function SHCompleteProcess64(const AApplication:string; var AParameters:string;
 var AExitCode:cardinal; AWaitInterval:dword=INFINITE; ANewConsole: boolean = false):dword;

implementation

function SHCompleteProcess(const ACommandLine:string; var AExitCode:cardinal;
 AWaitInterval:dword=INFINITE; ANewConsole: boolean = false):dword;
var
  LProcessHandle:THandle;
  StartupInfo:TStartupInfo;
  ProcessInformation:TProcessInformation;
  LCreationFlags: cardinal;
begin
  {Initiate process}
  FillChar(StartupInfo,Sizeof(StartupInfo),0);
  FillChar(ProcessInformation,Sizeof(ProcessInformation),0);
  LCreationFlags := CREATE_DEFAULT_ERROR_MODE+CREATE_NEW_PROCESS_GROUP;
  if ANewConsole then
    Inc(LCreationFlags, CREATE_NEW_CONSOLE) else  // Want DOS window
    Inc(LCreationFlags, CREATE_NO_WINDOW);        // Don't want DOS window

  if not CreateProcess(nil,pchar(ACommandLine),
   nil,nil,false,LCreationFlags,nil,
   nil,StartupInfo,ProcessInformation)then begin
    Result:=PROCESS_NOT_START;
    Exit;
  end;

  try
    {Wait for it to complete then get exit code}
    LProcessHandle:=ProcessInformation.hProcess;
    Result:=WaitForSingleObject(LProcessHandle,AWaitInterval);
    GetExitCodeProcess(LProcessHandle,AExitCode);
  finally
    CloseHandle(ProcessInformation.hProcess);
    CloseHandle(ProcessInformation.hThread);
  end;
end;

function SHCompleteProcess64(const AApplication:string; var AParameters:string;
 var AExitCode:cardinal; AWaitInterval:dword=INFINITE; ANewConsole: boolean = false):dword;
var
  LProcessHandle:THandle;
  StartupInfo:TStartupInfo;
  ProcessInformation:TProcessInformation;
  LCreationFlags: cardinal;
begin
  {Initiate process}
  FillChar(StartupInfo,Sizeof(StartupInfo),0);
  FillChar(ProcessInformation,Sizeof(ProcessInformation),0);
  LCreationFlags := CREATE_DEFAULT_ERROR_MODE+CREATE_NEW_PROCESS_GROUP;
  if ANewConsole then
    Inc(LCreationFlags, CREATE_NEW_CONSOLE) else  // Want DOS window
    Inc(LCreationFlags, CREATE_NO_WINDOW);        // Don't want DOS window

  if not CreateProcess(pchar(AApplication), pchar(AParameters),
   nil,nil,false,LCreationFlags,nil,
   nil,StartupInfo,ProcessInformation)then begin
    Result:=PROCESS_NOT_START;
    Exit;
  end;

  try
    {Wait for it to complete then get exit code}
    LProcessHandle:=ProcessInformation.hProcess;
    Result:=WaitForSingleObject(LProcessHandle,AWaitInterval);
    GetExitCodeProcess(LProcessHandle,AExitCode);
  finally
    CloseHandle(ProcessInformation.hProcess);
    CloseHandle(ProcessInformation.hThread);
  end;
end;

end.
